@@ -277,63 +277,3 @@ class BetaCristobalit(Pattern):
         block.zero()
 
         return block
-
-    ############
-    # Exterior #
-    ############
-    def exterior(self, error=10e-3):
-        """This function converts the exterior surface in direction of the
-        orientation to binding sites by adding oxygens to unsaturated silicon
-        atoms.
-
-        Parameters
-        ----------
-        error : float, optional
-            Tollerated error for position check
-        """
-        # Get silicon atoms
-        si_list = [atom_id for atom_id, atom in enumerate(self._structure.get_atom_list()) if atom.get_atom_type()=="Si"]
-        saturate = []
-
-        # Block oriented towards x-axis
-        if self._orient == "x":
-            is_left = False
-            for si in si_list:
-                if abs(self._structure.pos(si)[2]-self._size[2]+self._gap[2]/2) < error:
-                    saturate.append([si, is_left])
-
-        # Block oriented towards y-axis
-        elif self._orient == "y":
-            right = 0.413
-            left  = 0.826
-            for si in si_list:
-                pos = self._structure.pos(si)
-                if abs(pos[2]-self._size[2]+self._gap[2]*3/2) < error:
-                    if abs(pos[1]-right+self._gap[1]/2)%self._repeat[2] < error:
-                        saturate.append([si, False])
-                elif abs(pos[2]-self._gap[2]/2) < error:
-                    if abs(pos[1]-left+self._gap[1]/2)%self._repeat[2] < error:
-                        saturate.append([si, None])
-                    else:
-                        saturate.append([si, True])
-
-        # Block oriented towards z-axis
-        elif self._orient == "z":
-            is_left = True
-            for si in si_list:
-                if abs(self._structure.pos(si)[2]-self._gap[2]/2) < error:
-                    saturate.append([si, is_left])
-
-        # Saturate silicon atoms
-        for si in saturate:
-            if si[1] is not None:
-                r = -0.155 if si[1] else 0.155
-                self._structure.add("O", si[0], r=r)
-            else:
-                r = -0.155
-                self._structure.add("O", si[0], r=r, theta= 40)
-                self._structure.add("O", si[0], r=r, theta=-40)
-
-        # Move box to zero
-        self._structure.zero()
-        self._structure.translate(self._gap)
