@@ -43,6 +43,7 @@ class Pore():
         self._matrix = matrix
 
         self._num_in_ex = 0
+        self._oxygen_ex = []
 
         self._mol_dict = {"block": {}, "in": {}, "ex": {}}
 
@@ -69,10 +70,11 @@ class Pore():
                 self._matrix.strip(atom)
 
         # Remove silicon atoms with three unsaturated oxygen atoms
-        si_count = Counter(sum([self._matrix.get_matrix()[atom] for atom in self._matrix.bound(1)], []))
-        for si, count in si_count.items():
-            if count == 3:
-                self._matrix.strip(si)
+        while 3 in Counter(sum([self._matrix.get_matrix()[atom] for atom in self._matrix.bound(1)], [])).values():
+            si_count = Counter(sum([self._matrix.get_matrix()[atom] for atom in self._matrix.bound(1)], []))
+            for si, count in si_count.items():
+                if count >= 3:
+                    self._matrix.strip(si)
 
     def amorph(self, dist=0.05, accept=[0.1, 0.2], trials=100):
         """Make Structure amorphous.
@@ -117,7 +119,72 @@ class Pore():
                     self._block.get_atom_list()[atom_id].set_pos(disp_pos)
                     break
 
-    def sites(self, exterior=[]):
+    def exterior(self, gap):
+        """Create an exterior surface for reservoir attachement. Using the
+        periodic boundary distance as the indication, the bond between Si and O
+        is broken and the silicon atom is saturated with and additional O atom
+        using the original bond vector for orientation. At the end, the system
+        is shifted by the given gap vector which represents the distance between
+        bonds.
+
+        Parameters
+        ----------
+        gap : list
+            Gap between box boundary and crystal structure
+        """
+        # Initialize
+        box = self._block.get_box()
+        atom_list = self._block.get_atom_list()
+        bound_list = self._matrix.get_matrix()
+
+        # Get list of all si atoms
+        si_list = [atom_id for atom_id, atom in enumerate(self._block.get_atom_list()) if atom.get_atom_type()=="Si"]
+
+        # Run through silicon atoms
+        add_list = []
+        break_list = []
+        for si in si_list:
+            # Run through bound oxygen atoms
+            for o in bound_list[si]:
+                # Calculate bond vector
+                bond_vector = [atom_list[si].get_pos()[dim]-atom_list[o].get_pos()[dim] for dim in range(3)]
+
+                # Check if z dimension of bond - after rotation in pattern class - goes over boundary
+                if abs(bond_vector[2]) > box[2]/2:
+                    # Get bond direction
+                    r = -0.155 if bond_vector[2] < 0 else 0.155
+
+                    # Get bond angle
+                    theta = geometry.angle_azi(bond_vector, is_deg=True)
+                    phi = geometry.angle_polar(bond_vector, is_deg=True)
+                    theta = theta-180 if r < 0 else theta
+
+                    # Add new oxygen atom
+                    self._block.add("O", si, r=r, theta=theta, phi=phi)
+
+                    # Add list for new atoms
+                    add_list.append([si, self._block.get_num()-1])
+
+                    # Add to break bond list
+                    break_list.append([si, o])
+
+        # Move box to zero
+        self._block.zero()
+        self._block.translate(gap)
+
+        # Break periodic bonds
+        for bond in break_list:
+            self._matrix.split(bond[0], bond[1])
+
+        # Add bonds to new oxygens
+        for bond in add_list:
+            self._matrix.add(bond[0], bond[1])
+
+        # Add atoms to exterior oxygen list
+        self.prepare()
+        self._oxygen_ex = self._matrix.bound(1)
+
+    def sites(self):
         """Create binding site dictionary of the format
 
         .. math::
@@ -143,11 +210,6 @@ class Pore():
         * **o** - Unsaturated oxygen atoms bound to silicon atom
         * **type** - Exterior "ex" or interior "in" binding site
         * **state** - available - True, used - False
-
-        Parameters
-        ----------
-        exterior : list, optional
-            List of oxygen atom ids that are on the exterior surface
         """
         # Get list of surface oxygen atoms
         oxygen_list = self._matrix.bound(1)
@@ -165,14 +227,11 @@ class Pore():
             # Site type
             is_in = False
             is_ex = False
-            if exterior:
-                for o in data["o"]:
-                    if o in exterior:
-                        is_ex = True
-                    else:
-                        is_in = True
-            else:
-                is_in = True
+            for o in data["o"]:
+                if o in self._oxygen_ex:
+                    is_ex = True
+                else:
+                    is_in = True
 
             data["type"] = "ex" if is_ex else "in"
 
@@ -239,7 +298,7 @@ class Pore():
             mol_diam = (max(mol.get_box()[:2])+0.17)*scale
             si_atoms = [self._block.get_atom_list()[atom] for atom in sites]
             si_dice = Dice(Molecule(inp=si_atoms), mol_diam, True)
-            si_proxi = si_dice.find_parallel(None, ["Si", "Si"], 0, mol_diam)
+            si_proxi = si_dice.find_parallel(None, ["Si", "Si"], [-mol_diam, mol_diam])
             si_matrix = {x[0]: x[1] for x in si_proxi}
 
         # Run through number of binding sites to add
@@ -306,7 +365,7 @@ class Pore():
 
         return mol_list
 
-    def siloxane(self, sites, amount, normal, slx_dist=0.507, trials=1000, site_type="in"):
+    def siloxane(self, sites, amount, normal, slx_dist=[0.507-1e-2, 0.507+1e-2], trials=1000, site_type="in"):
         """Attach siloxane bridges on the surface similar to Krishna et al.
         (2009). Here silicon atoms of silanol groups wich are at least 0.31 nm
         near each other can be converted to siloxan bridges, by removing one
@@ -322,8 +381,9 @@ class Pore():
         normal : function
             Function that returns the normal vector of the surface for a given
             position
-        slx_dist : float
-            Silicon atom distance to search for parters in proximity
+        slx_dist : list
+            Silicon atom distance bounds to search for parters in proximity
+            [lower, upper]
         trials : integer, optional
             Number of trials picking a random site
         site_type : string, optional
@@ -348,23 +408,36 @@ class Pore():
 
         # Search for silicon atoms near each other
         si_atoms = [self._block.get_atom_list()[atom] for atom in sites]
-        si_dice = Dice(Molecule(inp=si_atoms), slx_dist+0.1, True)
-        si_proxi = si_dice.find_parallel(None, ["Si", "Si"], slx_dist, 1e-2)
+        si_dice = Dice(Molecule(inp=si_atoms), slx_dist[1], False)
+        si_proxi = si_dice.find_parallel(None, ["Si", "Si"], slx_dist)
         si_matrix = {x[0]: x[1] for x in si_proxi}
 
         # Run through number of siloxan bridges to add
+        bond_matrix = self._matrix.get_matrix()
         mol_list = []
         for i in range(amount):
             # Randomly pick an available site pair
             si = []
             for j in range(trials):
                 si_rand = random.choice(sites)
+                # Check if binding site in local si-si matrix and if it contains binding partners
                 if sites.index(si_rand) in si_matrix and si_matrix[sites.index(si_rand)]:
+                    # Choose first binding partner in list
                     si_rand_proxi = sites[si_matrix[sites.index(si_rand)][0]]
+                    # Check if binding partner is in local si-si matrix
                     if sites.index(si_rand_proxi) in si_matrix:
-                        if self._sites[si_rand]["state"] and (self._sites[si_rand_proxi]["state"]):
-                            si = [si_rand, si_rand_proxi]
-                            break
+                        # Check if unbound states
+                        if self._sites[si_rand]["state"] and self._sites[si_rand_proxi]["state"]:
+                            # Check if binding site silicon atoms are already connected with an oxygen
+                            is_connected = False
+                            for atom_o in bond_matrix[si_rand]:
+                                if atom_o in bond_matrix[si_rand_proxi]:
+                                    is_connected = True
+                            # Add to siloxane list if not connected
+                            if not is_connected:
+                                si = [si_rand, si_rand_proxi]
+                                # End trials if appended
+                                break
 
             # Place molecule on surface
             if si and self._sites[si[0]]["state"] and self._sites[si[1]]["state"]:
